package com.sun.org.apache.xerces.internal.dom;

import org.w3c.dom.Node;
import org.w3c.dom.DOMException;
import org.w3c.dom.NodeList;

public abstract class CharacterDataImpl extends ChildNode
{
    static final long serialVersionUID = 7931170150428474230L;
    protected String data;
    private static transient NodeList singletonNodeList;
    
    public CharacterDataImpl() {
    }
    
    protected CharacterDataImpl(CoreDocumentImpl ownerDocument, String data) {
        super(ownerDocument);
        this.data = data;
    }
    
    @Override
    public NodeList getChildNodes() {
        return CharacterDataImpl.singletonNodeList;
    }
    
    @Override
    public String getNodeValue() {
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        return this.data;
    }
    
    protected void setNodeValueInternal(String value) {
        this.setNodeValueInternal(value, false);
    }
    
    protected void setNodeValueInternal(String value, boolean replace) {
        CoreDocumentImpl ownerDocument = this.ownerDocument();
        if (ownerDocument.errorChecking && this.isReadOnly()) {
            String msg = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "NO_MODIFICATION_ALLOWED_ERR", (Object[])null);
            throw new DOMException((short)7, msg);
        }
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        String oldvalue = this.data;
        ownerDocument.modifyingCharacterData(this, replace);
        ownerDocument.modifiedCharacterData(this, oldvalue, this.data = value, replace);
        return;
    }
    
    @Override
    public void setNodeValue(String value) {
        this.setNodeValueInternal(value);
        this.ownerDocument().replacedText(this);
    }
    
    public String getData() {
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        return this.data;
    }
    
    @Override
    public int getLength() {
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        return this.data.length();
    }
    
    public void appendData(String data) {
        if (this.isReadOnly()) {
            String msg = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "NO_MODIFICATION_ALLOWED_ERR", (Object[])null);
            throw new DOMException((short)7, msg);
        }
        if (data == null) {
            return;
        }
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        this.setNodeValue(this.data + data);
        return;
    }
    
    public void deleteData(int offset, int count) throws DOMException {
        this.internalDeleteData(offset, count, false);
    }
    
    void internalDeleteData(int offset, int count, boolean replace) throws DOMException {
        CoreDocumentImpl ownerDocument = this.ownerDocument();
        if (ownerDocument.errorChecking) {
            if (this.isReadOnly()) {
                String msg = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "NO_MODIFICATION_ALLOWED_ERR", (Object[])null);
                throw new DOMException((short)7, msg);
            }
            if (count < 0) {
                String msg = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "INDEX_SIZE_ERR", (Object[])null);
                throw new DOMException((short)1, msg);
            }
        }
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        int tailLength = Math.max(this.data.length() - count - offset, 0);
        try {
            String value = this.data.substring(0, offset) + ((tailLength > 0) ? this.data.substring(offset + count, offset + count + tailLength) : "");
            this.setNodeValueInternal(value, replace);
            ownerDocument.deletedText(this, offset, count);
        }
        catch (StringIndexOutOfBoundsException e) {
            String msg2 = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "INDEX_SIZE_ERR", (Object[])null);
            throw new DOMException((short)1, msg2);
        }
    }
    
    public void insertData(int offset, String data) throws DOMException {
        this.internalInsertData(offset, data, false);
    }
    
    void internalInsertData(int offset, String data, boolean replace) throws DOMException {
        CoreDocumentImpl ownerDocument = this.ownerDocument();
        if (ownerDocument.errorChecking && this.isReadOnly()) {
            String msg2 = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "NO_MODIFICATION_ALLOWED_ERR", (Object[])null);
            throw new DOMException((short)7, msg2);
        }
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        try {
            String value = new StringBuffer(this.data).insert(offset, data).toString();
            this.setNodeValueInternal(value, replace);
            ownerDocument.insertedText(this, offset, data.length());
        }
        catch (StringIndexOutOfBoundsException e) {
            String msg = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "INDEX_SIZE_ERR", (Object[])null);
            throw new DOMException((short)1, msg);
        }
        return;
    }
    
    public void replaceData(int offset, int count, String data) throws DOMException {
        CoreDocumentImpl ownerDocument = this.ownerDocument();
        if (ownerDocument.errorChecking && this.isReadOnly()) {
            String msg = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "NO_MODIFICATION_ALLOWED_ERR", (Object[])null);
            throw new DOMException((short)7, msg);
        }
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        ownerDocument.replacingData(this);
        String oldvalue = this.data;
        this.internalDeleteData(offset, count, true);
        this.internalInsertData(offset, data, true);
        ownerDocument.replacedCharacterData(this, oldvalue, this.data);
        return;
    }
    
    public void setData(String value) throws DOMException {
        this.setNodeValue(value);
    }
    
    public String substringData(int offset, int count) throws DOMException {
        if (this.needsSyncData()) {
            this.synchronizeData();
        }
        int length = this.data.length();
        if (count < 0 || offset < 0 || offset > length - 1) {
            String msg = DOMMessageFormatter.formatMessage("http://www.w3.org/dom/DOMTR", "INDEX_SIZE_ERR", (Object[])null);
            throw new DOMException((short)1, msg);
        }
        int tailIndex = Math.min(offset + count, length);
        return this.data.substring(offset, tailIndex);
    }
    
    static {
        CharacterDataImpl.singletonNodeList = new NodeList() {
            @Override
            public Node item(int index) {
                return null;
            }
            
            @Override
            public int getLength() {
                return 0;
            }
        };
    }
}
